// CHANGED 2004-07-28 Lutz Rogowski
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <g711.h>
#include <qtimer.h>


#include "../dissipate2/sipuri.h"
#include "../dissipate2/sipclient.h"
#include "audiobuffer.h"
#include "rtpdataheader.h"
#include "dspoutrtp.h"
#include "kphoneconst.h"
#include "kcallsipsettings.h"

#include <kdebug.h>

#define BLOCKL  BLOCKL_30MS
#define NO_OF_BYTES NO_OF_BYTES_30MS

short ILBCencode( iLBC_Enc_Inst_t *iLBCenc_inst,short *encoded_data, short *data);
short ILBCdecode( iLBC_Dec_Inst_t *iLBCdec_inst,short *decoded_data, short *encoded_data, short mode);

extern unsigned char s2u[];
extern unsigned short u2s[] ;

DspOutRtp::DspOutRtp( const codecType newCodec, int newCodecNum, const QString &hostName, UDPMessageSocket *s )
{
        codec = newCodec;
	codecNum = newCodecNum;
	if (s) {
		socket = s;
		destroySocket = false;
	} else {
		socket = new UDPMessageSocket();
		destroySocket = true;
	}
	if (socket == 0) {
		printf("DspOutRtp:: Can't create socket\n");
	}
	if( hostName != QString::null && !socket->setHostname( hostName.latin1() ) ) {
		printf( "DspOutRtp::Hostname lookup failed\n" );
	}
	dsize = 4096*2*sizeof( short );  // initial max packet size
    createBuffer( dsize );

	if( !( gsmInstEnc = gsm_create() ) ) {
		printf( "DspOutRtp::GSM_CREATE -error !\n" );
	}
	if( !( gsmInstDec = gsm_create() ) ) {
		printf( "DspOutRtp::GSM_CREATE -error !\n" );
	}
	mqlen = 0;
	initEncode(&ilbcEncInst, 30 );
	initDecode(&ilbcDecInst, 30, 1);
	numunsend = 0;
	fixedrtplen = 160;
	curseq = 0;
	ts = rand();
	ssrc = 10;
	ref_sec = 0;
	ref_usec = 0;
	portnum = 0;
	audio_buf.resize( dsize );
	setDeviceName( "rtp" );
	deb_frag=0L;
	deb_rtp=0L;
	useStun = false;
    maxBufferSize = dsize;
}

DspOutRtp::~DspOutRtp( void )
{
	if( gsmInstEnc ) gsm_destroy( gsmInstEnc );
	if( gsmInstDec ) gsm_destroy( gsmInstDec ); 
    deleteBuffer();
	if (destroySocket) delete socket;
    qDebug("~DspOutRtp::MaxBufferSize: %d",maxBufferSize );
}

bool DspOutRtp::openDevice( DeviceMode mode )
{
	int min, max;
	min = KCallSipSettings::minPort();
	max = KCallSipSettings::maxPort();
	devmode = mode;
	if( devmode == ReadOnly ) {
		socket->listenOnEvenPort(min, max);
		portnum = socket->getPortNumber();
		/* Use STUN to rewrite port number only if the user
		 * didn't set a static port range which is being forwarded
		 * on the NAT
		 */
		if ((min == 0) && (max == 0)) {
			if ( sendStunRequest( socket ) ) {
				portnum = receiveStunResponse( socket );
				if( portnum > 0 ) {
					socket->forcePortNumber( portnum );
				}
			}
		}

//------------------------------
// Linux kernels 2.4.x
// Do not allow any double binds even when same user and REUSEADDR.
/*
		UDPMessageSocket *socketVideo = new UDPMessageSocket() ;
		socketVideo->forcePortNumber( videoPortnum );
		socketVideo->listenOnEvenPortForVideo();
		videoPortnum = socketVideo->getPortNumber();
		socketVideo->forcePortNumber( videoPortnum );
		if( sendStunRequest( *socketVideo ) ) {
			videoPortnum = receiveStunResponse( *socketVideo );
		}
		delete socketVideo;
*/
		videoPortnum = portnum + 20;
//------------------------------

	} else {
		socket->connect( portnum );
		socket->SetTOS();
	}
	devstate = DeviceOpened;
	return true;
}

void DspOutRtp::deleteBuffer()
{

	delete[] static_cast<unsigned char *>( packetbuf );
	delete[] static_cast<unsigned char *>( bufunsend );
	delete[] static_cast<unsigned char *>( outbuf );
   	delete[] static_cast<unsigned char *>( tmpbuf );
	delete[] static_cast<unsigned char *>( quebuf );

}
void DspOutRtp::createBuffer( int size )
{

	packetbuf = new unsigned char[ sizeof( rtp_hdr_t ) + size ];
	bufunsend = new unsigned char[ size ];
	outbuf = new unsigned char[ size ];
	tmpbuf = new unsigned char[ size ];
	quebuf = new unsigned char[ size ];
}
bool DspOutRtp::writeBuffer( void )
{
	short *s;
	short *frombuf;
	unsigned char *databuf;
	rtp_hdr_t *h = (rtp_hdr_t *) packetbuf;
	size_t fromsize;
	unsigned char *wlbuf;
	size_t bytesnew;
	size_t bytesgot;
	int tmpsize;
	unsigned int i;
	bytesnew = audio_buf.getSize() / sizeof( short );
	fromsize = audio_buf.getSize() / sizeof( short );
    if ( audio_buf.getSize() > maxBufferSize ) {
        if ( audio_buf.getSize() > 64000 ) {
            qDebug("Audiobuffer too big! ");
            qDebug("Skipping audio stream! ");
            return false;
        }
        maxBufferSize = audio_buf.getSize()+1024;
        qDebug("DspOutRtp::Resizing audio buffers to %d",maxBufferSize );
        deleteBuffer();
        createBuffer(maxBufferSize );
    }
	inbuf = (unsigned char*)audio_buf.getData();
	frombuf = (short *)inbuf;
	if( codec == codecGSM ) {
		short *frombuf_test;
		frombuf_test = frombuf;
		s = (short *)inbuf;
		for( i=0; i<fromsize; i++) {
			s[i] = (short)(*frombuf_test);
			frombuf_test++;
		}
		tmpsize = writeGSMBuffer( gsmInstEnc, inbuf, outbuf, tmpbuf, quebuf, &mqlen, fromsize );
		while( tmpsize > 0 ) {
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = 3;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += GSM_DEC_SAMPLES;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
            unsigned char *outbufit = outbuf;
			for( int i = 0; i < tmpsize; i++ ) {
				*databuf++ = *outbufit++;
			}
			if( socket->send( (char *) packetbuf, sizeof( rtp_hdr_t ) + tmpsize ) < 0 ) {
				printf("DspOutRtp::writeBuffer: %s\n", strerror(errno));
				return false;
			}
			tmpsize = writeGSMBuffer( gsmInstEnc, inbuf, outbuf, tmpbuf, quebuf, &mqlen, 0 );
		}
	} else if( codec == codecILBC ) {
		short *frombuf_test;
		frombuf_test = frombuf;
		s = (short *)inbuf;
		for( i=0; i<fromsize; i++) {
			s[i] = (short)(*frombuf_test);
			frombuf_test++;
		}
        int allsize = 0;
		tmpsize = writeILBCBuffer( &ilbcEncInst, inbuf, outbuf, tmpbuf, quebuf, &mqlen, fromsize );
		while( tmpsize > 0 ) {
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = 97 ;//codecNum;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += BLOCKL;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
            unsigned char *outbufit = outbuf;
			for( int i = 0; i < tmpsize; i++ ) {
				*databuf++ = *outbufit++;
			}
            allsize += tmpsize;
            //qDebug("ILBCB %d send %d",curseq, tmpsize );
			if( socket->send( (char *) packetbuf, sizeof( rtp_hdr_t ) + tmpsize ) < 0 ) {
				printf("DspOutRtp::writeBuffer: %s\n", strerror(errno));
				return false;
			}
			tmpsize = writeILBCBuffer( &ilbcEncInst, inbuf, outbuf, tmpbuf, quebuf, &mqlen, 0 );
		}
	} else if( codec == codecULAW ) {
		wlbuf = bufunsend;
		while(bytesnew+numunsend >= fixedrtplen){
			deb_rtp++;
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = 0;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += fixedrtplen;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
			bytesgot = 0;
			while( numunsend > 0 ) {
				*databuf = *wlbuf;
				++databuf;
				++wlbuf;
				--numunsend;
				++bytesgot;
			}
			wlbuf = bufunsend;
			while( bytesgot < fixedrtplen ) {
				*databuf = s2u[ ((unsigned short)(*frombuf)) >> 3 ];//linear2ulaw( (short) (*frombuf) );
				++databuf;
				++frombuf;
				--bytesnew;
				++bytesgot;
			}
			if( socket->send( (char *) packetbuf, sizeof( rtp_hdr_t ) + fixedrtplen ) < 0 ) {
				printf("DspOutRtp::writeBuffer: %s\n", strerror(errno));
				return false;
			}
		}

		if( bytesnew > 0 ) {
			wlbuf = bufunsend + numunsend;
			numunsend += bytesnew;
			while( bytesnew > 0 ){
				*wlbuf = s2u[ ((unsigned short)(*frombuf)) >> 3 ];//linear2ulaw( (short) (*frombuf) );
				++wlbuf;
				++frombuf;
				--bytesnew;
			}
		}
		deb_frag++;
	}

	return true;
}

bool DspOutRtp::setPortNum( int newport )
{
	portnum = newport;
	return true;
}

unsigned int DspOutRtp::readableBytes( void )
{
    //qDebug("DspOutRtp::readableBytes ");
	struct timeval timeout;
	fd_set read_fds;
	int highest_fd;
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	FD_ZERO( &read_fds );
	FD_SET( socket->getFileDescriptor(), &read_fds );
	highest_fd = socket->getFileDescriptor() + 1;
retry:
	if ( select( highest_fd, &read_fds, NULL, NULL, &timeout ) == -1 ) {
		if ( errno == EINTR ) goto retry;
		perror( "DspOutRtp::doSelect(): select() punted" );
		exit( 1 );
	}
	if ( FD_ISSET( socket->getFileDescriptor(), &read_fds ) ) {
		return 1;
	}
	return 0;
}

int DspOutRtp::readBuffer( int bytes )
{
	unsigned char *inbuf;
	short *outbuf;
	int recvsize;
	int i;
	int size;
    static ushort lastseq = 2000; 
	recvsize = socket->receive( (char *) packetbuf, sizeof( rtp_hdr_t ) + dsize );
	rtp_hdr_t *h = (rtp_hdr_t *) packetbuf;

    //qDebug("readBuffer %d ",ntohs( h->seq ) );
	if( h->pt == 0 ) {
        //qDebug("resize %d %d %d",(int) sizeof( rtp_hdr_t ) ,recvsize, h->seq );
        int fac = 1;
        ushort sequ = ntohs( h->seq );
        if ( !(sequ < 100 && lastseq > 1000) ) {
            if (  lastseq + 1 < sequ) {
                ++fac;
                if (   lastseq + 2 < sequ )
                    ++fac;
                //qDebug("lost %d",sequ );
            }
        }
        lastseq = sequ;
		audio_buf.resize( ( recvsize - (int) sizeof( rtp_hdr_t ) ) * sizeof( short ) *fac);
		outbuf = (short *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
        int max = recvsize - (int) sizeof( rtp_hdr_t );
		for( i = 0; i < max; i++ ) {
			*outbuf++ = (short)u2s[*inbuf++];//(short) ulaw2linear( inbuf[ i ] );
        }
        --fac;
        if ( fac ) {
            //qDebug("fac %d ",lastseq );
            memcpy ( (void*)outbuf, audio_buf.getData(), max*sizeof( short )); 
            --fac;
            if ( fac ) 
                memcpy ( (void*) (outbuf+max*sizeof( short )) , audio_buf.getData(), max*sizeof( short ));
        }

	} else if( h->pt == 3 ) { 
        audio_buf.resize( GSM_DEC_SAMPLES * sizeof( short ) );
		outbuf = (short *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
		readGSMBuffer( gsmInstDec, inbuf, tmpbuf, false );
		short *s = (short *)tmpbuf;
		for( i = 0; i < GSM_DEC_SAMPLES; i++) {
			*outbuf++ = (short)(s[i]);
		}
	} else if( codec == codecILBC && (int)h->pt == codecNum ) {
        //qDebug("READ %d ilc %d ",ntohs(h->seq), recvsize-sizeof( rtp_hdr_t ) );
		audio_buf.resize( BLOCKL * sizeof( short ) );
		outbuf = (short *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
		size = readILBCBuffer( &ilbcDecInst, inbuf, tmpbuf, 1, false );
		short *s = (short *)tmpbuf;
		for( i = 0; i < size; i++) {
			*outbuf++ = (short)(s[i]);
		}
	}
	return 1;
}

bool DspOutRtp::sendStunRequest( UDPMessageSocket *socketStun )
{
	SipUri stun;
	if( useStun ) {
		stun = SipUri( stunSrv );
		if( !socketStun->setHostname( stun.getHostname() ) ) {
			return false;
		}
		socketStun->connect( stun.getPortNumber() );
		printf( "DspOutRtp: STUN request\n" );
		StunRequestSimple req;
		req.msgHdr.msgType = htons(BindRequestMsg);
		req.msgHdr.msgLength = htons( sizeof(StunRequestSimple)-sizeof(StunMsgHdr) );
		for ( int i=0; i<16; i++ ) {
			req.msgHdr.id.octet[i]=0;
		}
		int id = rand();
		req.msgHdr.id.octet[0] = id;
		req.msgHdr.id.octet[1] = id>>8;
		req.msgHdr.id.octet[2] = id>>16;
		req.msgHdr.id.octet[3] = id>>24;
		socketStun->send( (char *)&req, sizeof( req ) );
	} else {
		return false;
	}
	return true;
}

unsigned int DspOutRtp::receiveStunResponse( UDPMessageSocket *socketStun )
{
	unsigned int port = 0;
	char inputbuf[ 8000 ];
	int bytesread;
	struct timeval timeout;
	fd_set read_fds;
	int highest_fd;
	timeout.tv_sec = 5;
	timeout.tv_usec = 0;
	FD_ZERO( &read_fds );
	FD_SET( socketStun->getFileDescriptor(), &read_fds );
	highest_fd = socketStun->getFileDescriptor() + 1;
retry:
	if( select( highest_fd, &read_fds, NULL, NULL, &timeout ) == -1 ) {
		if( errno == EINTR ) goto retry;
		perror( "SipClient::doSelect(): select() punted" );
		exit( 1 );
	}
	if( FD_ISSET( socketStun->getFileDescriptor(), &read_fds ) ) {
		printf( "SipClient: Receiving message...\n" );
		bytesread = read( socketStun->getFileDescriptor(), inputbuf, 8000 - 1 );
		StunMsgHdr* hdr = reinterpret_cast<StunMsgHdr*>( inputbuf );
		if( hdr->msgType == BindResponseMsg ) {
			printf( "SipClient: STUN response for RTP\n" );
			char* body = inputbuf + sizeof( StunMsgHdr );
			unsigned int size = ntohs( hdr->msgLength );
			while( size > 0 ) {
				StunAtrHdr* attr = reinterpret_cast<StunAtrHdr*>( body );
				unsigned int hdrLen = ntohs( attr->length );
				if( ntohs( attr->type ) == MappedAddress ) {
					StunAtrAddress* attribute =
						reinterpret_cast<StunAtrAddress*>( body );
					if ( attribute->address.addrHdr.family == IPv4Family ) {
						StunAtrAddress4* atrAdd4 =
							reinterpret_cast<StunAtrAddress4*>( body );
						if ( hdrLen == sizeof( StunAtrAddress4 ) - 4 ) {
							port = ntohs( atrAdd4->addrHdr.port );
						}
					}
				}
				body += hdrLen+4;
				size -= hdrLen+4;
			}
		}
	}
	return port;
}

void DspOutRtp::setCodec( const codecType newCodec, int newCodecNum )
{
	codec = newCodec;
	codecNum = newCodecNum;
}

/************************************************************************/
/* GSM                                                                   */
/************************************************************************/

int DspOutRtp::writeGSMBuffer( gsm gsmInst,
		unsigned char *input_buf, unsigned char *output_buf,
		unsigned char *tmp_buf, unsigned char *queue,
		int *qlen, int size )
{
	int outbufsize = 0;
	int inq, inx;
	short *ibuf;
	short *obuf;
	short *tbuf;
	short *qbuf;
	ibuf = (short *)input_buf;
	obuf = (short *)output_buf;
	qbuf = (short *)queue;
	inq = *qlen;
	inx = 0;
	if( inq >= GSM_DEC_SAMPLES ) {
		tbuf = (short *)tmp_buf;
		while( inx < GSM_DEC_SAMPLES ) {
			*tbuf = *qbuf;
			++tbuf;
			++qbuf;
			--inq;
			++inx;
		}
		tbuf = (short *)tmp_buf;
		gsm_encode( gsmInst, (gsm_signal *)tbuf, (gsm_byte *)obuf);
		*qlen = inq;
		outbufsize = GSM_ENC_BYTES;
	} else if( inq + size >= GSM_DEC_SAMPLES ) {
		tbuf = (short *)tmp_buf;
		while( inq > 0 ) {
			*tbuf = *qbuf;
			++tbuf;
			++qbuf;
			--inq;
			++inx;
		}
		*qlen = 0;
		while( inx < GSM_DEC_SAMPLES ) {
			*tbuf = *ibuf;
			++tbuf;
			++ibuf;
			--size;
			++inx;
		}
       
		tbuf = (short *)tmp_buf;
		gsm_encode( gsmInst, (gsm_signal *)tbuf, (gsm_byte *)obuf);
		outbufsize = GSM_ENC_BYTES;

	}
	if( size > 0 ) {
		inq = *qlen;
		qbuf = (short *)(queue + ( inq * sizeof(short) ) );
		while( size > 0 ) {
			*qbuf = *ibuf;
			++inq;
			--size;
			++qbuf;
			++ibuf;
			++inx;
		}
        
		*qlen = inq;
	}
	return outbufsize;
}

int DspOutRtp::readGSMBuffer( gsm Gsm_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		int ignore )
{
	short *outbuf;
	short *ibuf;
	outbuf = (short *)output_buf;
	ibuf = (short *)input_buf;
	if( !ignore ) {
		gsm_decode( Gsm_Inst, (gsm_byte *)ibuf, (gsm_signal *)outbuf );
	}
	return true;
}



/*************************************************************************/
/* iLBC                                                                   */
/*************************************************************************/

int DspOutRtp::writeILBCBuffer( iLBC_Enc_Inst_t *Enc_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		unsigned char *tmp_buf, unsigned char *queue,
		int *qlen, int size )
{
	int outbufsize = 0;
	int inq, inx;
	short *ibuf;
	short *obuf;
	short *tbuf;
	short *qbuf;
	ibuf = (short *)input_buf;
	obuf = (short *)output_buf;
	qbuf = (short *)queue;
	inq = *qlen;
	inx = 0;
	if( inq >= BLOCKL ) {
		tbuf = (short *)tmp_buf;
		while( inx < BLOCKL ) {
			*tbuf = *qbuf;
			++tbuf;
			++qbuf;
			--inq;
			++inx;
		}
		tbuf = (short *)tmp_buf;
		outbufsize = ILBCencode( Enc_Inst, obuf, tbuf);
		*qlen = inq;
	} else if( inq + size >= BLOCKL ) {
		tbuf = (short *)tmp_buf;
		while( inq > 0 ) {
			*tbuf = *qbuf;
			++tbuf;
			++qbuf;
			--inq;
			++inx;
		}
		*qlen = 0;
		while( inx < BLOCKL ) {
			*tbuf = *ibuf;
			++tbuf;
			++ibuf;
			--size;
			++inx;
		}
		tbuf = (short *)tmp_buf;
		outbufsize = ILBCencode( Enc_Inst, obuf, tbuf);
	}
	if( size > 0 ) {
		inq = *qlen;
		qbuf = (short *)(queue + ( inq * sizeof(short) ) );
		while( size > 0 ) {
			*qbuf = *ibuf;
			++inq;
			--size;
			++qbuf;
			++ibuf;
		}
		*qlen = inq;
	}
	return outbufsize;
}


int DspOutRtp::readILBCBuffer( iLBC_Dec_Inst_t *Dec_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		short mode, int ignore)
{
	short *outbuf;
	short *ibuf;
	outbuf = (short *)output_buf;
	ibuf = (short *)input_buf;
	if( !ignore ) {
		return ILBCdecode( Dec_Inst, outbuf, ibuf, mode );
	}
	return 0;
}


void  DspOutRtp::setStunSrv( QString newStunSrv )
{
	useStun = true;
	stunSrv = newStunSrv;
}

/*----------------------------------------------------------------*
 *  Encoder interface function
 *---------------------------------------------------------------*/

short ILBCencode( iLBC_Enc_Inst_t *iLBCenc_inst,
		short *encoded_data,short *data )
{
	float block[BLOCKL];
	int k;

/* convert signal to float */

	for(k=0;k<BLOCKL;k++) block[k] = (float)data[k];

/* do the actual encoding */

	iLBC_encode((unsigned char *)encoded_data,block,iLBCenc_inst);

	return (short)(NO_OF_BYTES);
}


/*----------------------------------------------------------------*
 *  Decoder interface function
 *---------------------------------------------------------------*/

short ILBCdecode(iLBC_Dec_Inst_t *iLBCdec_inst,
		short *decoded_data, short *encoded_data, short mode)
{
	int k;
	float decblock[BLOCKL], dtmp;

/* check if mode is valid */

	if(mode<0 || mode>1){
		printf("\nERROR - Wrong mode - 0, 1 allowed\n");
		exit(3);
	}

/* do actual decoding of block */

	iLBC_decode( decblock,
		(unsigned char *)encoded_data, iLBCdec_inst, mode);

/* convert to short */

	for(k=0;k<BLOCKL;k++){
		dtmp=decblock[k];
		if (dtmp<MIN_SAMPLE)dtmp=MIN_SAMPLE;
		else if (dtmp>MAX_SAMPLE)dtmp=MAX_SAMPLE;
		decoded_data[k] = (short) dtmp;
	}

	return (short)BLOCKL;
}


