/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
//This file was modified by Mike Hauth and Eva Brucherseifer



#ifndef CALLMANAGER_H
#define CALLMANAGER_H

#include <qobject.h>
#include <qptrdict.h>
#include <qevent.h>
#include <time.h>
#include <kconfig.h>
#include <kstandarddirs.h>

#include "../dissipate2/sipuri.h"
//#include "sharp_char.h"



class SipUser;
class SipCall;
class SipCallMember;
class SipTransaction;
class CallAudio;
class IncomingCall;
class SipMessage;
class KSipAuthentication;

namespace AudioStatus
{
enum {
    Unattached,
    Active,
    OnHold
};
}


class CallManager : public QObject
{
    Q_OBJECT
public:
    CallManager( KSipAuthentication *auth, CallAudio *callaudio,
                 SipCall *initcall, bool incoming,
                 QWidget *parent = 0, const char *name = 0 );
    ~CallManager( void );
    SipCall *getCall();
    void switchCall( SipCall *newcall );
    void setRemote( QString newremote );

    void prepareForDelete();

public slots:
    void dial( void );
    void hangupCall( void );
    void acceptCall( void );
    void holdCall( void );

signals:
    void redirectCall( const SipUri &calluri, const QString &subject );
    void readyForDelete( const QString& callid);

private slots:
    void audioOutputDead( void );
    void callMemberStatusUpdated( void );
    void updateAudioStatus( void );
    void handleRedirect( void );
    void handleRefer( void );
    void pleaseTransfer( const QString &transferto );
//    void ringTimeout( void );
    void acceptCallTimeout( void );
//    void stopRing();
//    void startRing();

protected slots:
    //	void keypadPressed( int id );
    //	void keypadReleased( int id );

private:
    //    sharp_led_status statusLED;
    //	IncomingCall *incomingCall;
    void forceDisconnect( void );
    
    // Sip Stuff
    SipCall *call;
    SipCallMember *member;
    KSipAuthentication *sipauthentication;
    QString remote;

    // Audio Stuff
    CallAudio *audio;

//    int ringCount;
//    QTimer *ringTimer;
//    bool isRingingTone;
    QTimer *acceptCallTimer;
    int audio_fd;
    QString subject;

//    bool useWav;
    bool isIncomingCall;
    
    // State stuff
    enum CallState {
        PreDial,
        Calling,
        Connected,
        Called };
    CallState curstate;
};

#endif // KCALLWIDGET_H_INCLUDED
