/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
//This file was modified by Eva Brucherseifer
/** copied from kphone */



#ifndef KSIPAUTHENTICATION_H_INCLUDED
#define KSIPAUTHENTICATION_H_INCLUDED
#include <qdialog.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qbutton.h>
#include <qlabel.h>
#include <qlineedit.h>

#include "sipcall.h"
#include "sipregister.h"

/** @ingroup callmanager 
  * @class KSipAuthenticationRequest 
  * This dialog asks for a password to pass to the sip server for authentication.
  */
class KSipAuthenticationRequest : public QDialog
{
    Q_OBJECT
public:
    KSipAuthenticationRequest( const QString &server, const QString &sipuri,
                               const QString &caption, QWidget *parent = 0, const char *name = 0 );
    ~KSipAuthenticationRequest( void );
    void setUsername( const QString &newUsername );
    void setPassword( const QString &newPassword );
    QString username( void );
    QString password( void );

private:
    QLineEdit *usernameLineEdit;
    QLineEdit *passwordLineEdit;
    QPushButton *okPushButton;
    QPushButton *cancelPushButton;
};

/** @ingroup callmanager
  * @class KSipAuthentication
  * Holds authentication data for the sip server. If needed, it brings up a dialog
  * to ask for the password.
  */
class KSipAuthentication : public QObject
{
    Q_OBJECT
public:
    KSipAuthentication( void );
    ~KSipAuthentication( void );

    void setAuthData(const QString& user, const QString& pw);
    QStringList authData(const QString &server, const QString &sipuri, const QString &caption);
    QStringList newAuthData(const QString &server, const QString &sipuri, const QString &caption);
    
public slots:
    void authRequest( SipCallMember *member );

private:
//    void setRegisterState( void );
//    void save( void );
//    SipClient *c;
    //	KSipIdentityEdit *edit;
    //	KPhoneView *v;
//    QString userPrefix;
//    SipUser *u;
//    SipRegister *sipreg;
//    bool autoRegister;
//    int expires;
//    bool useStun;
//    QString stunSrv;   
    bool execAuthreq;
    
    QString username;
    QString password;
    bool authFromFile;
};

#endif // KSIPAUTHENTICATION_H_INCLUDED
