
#include "kcallgui_part.h"
#include "kcallguicore.h"
#include "kcallguimain.h"

#include "kcallsettings.h"
#include "kcallsipsettings.h"
#include "sipprefs.h"

#include <kinstance.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kfiledialog.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfigdialog.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qmultilineedit.h>

KCallGuiPart::KCallGuiPart( QWidget *parentWidget, const char *widgetName,
                                  QObject *parent, const char *name )
    : KParts::ReadWritePart(parent, name)
{
    kdDebug() << "KCallGuiPart::KCallGuiPart()" << endl;
    
    setInstance( KCallGuiPartFactory::instance() );
    
    mainWidget = new KCallGuiMain(parentWidget, widgetName);
    setWidget( mainWidget );
    mGuiCore = new KCallGuiCore(this, mainWidget);
     
    connect(mainWidget, SIGNAL(dial(QString)),
            mGuiCore, SLOT(dial(QString)));
    
    connect(mainWidget, SIGNAL(accept(QString)),
            mGuiCore, SLOT(acceptCall(QString)));
    
    //connect(mainWidget, SIGNAL(hold(QString)),
    //        mGuiCore, SLOT(holdCall(QString)));
    
    connect(mainWidget, SIGNAL(hangup(QString)),
            mGuiCore, SLOT(hangupCall(QString)));
            
    connect(mainWidget, SIGNAL(reject(QString)), 
            mGuiCore, SLOT(rejectCall(QString)));
    
    mGuiCore->getKCallStatus();
    
    // create our actions
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
    KStdAction::save(this, SLOT(save()), actionCollection());

//    if ( mIsPart )
    new KAction( i18n( "&Configure Phone..." ), "configure", 0,
                          this, SLOT( configure() ), actionCollection(),
                          "kcall_configure" );
//    else
//    KStdAction::preferences( this, SLOT( configure() ), actionCollection() );

    // set our XML-UI resource file
    setXMLFile("kcall_part.rc");

    // we are read-write by default
    //setReadWrite(true);

    // we are not modified since we haven't done anything yet
    //setModified(false);
   
}

KCallGuiPart::~KCallGuiPart()
{
    kdDebug() << "KCallGuiPart::~KCallGuiPart()" << endl;
    
    delete mGuiCore;
}

void KCallGuiPart::setReadWrite(bool rw)
{
    kdDebug() << "KCallGuiPart::setReadWrite()" << endl;
    // notify your internal widget of the read-write state
    m_widget->setReadOnly(!rw);
    if (rw)
        connect(m_widget, SIGNAL(textChanged()),
                this,     SLOT(setModified()));
    else
    {
        disconnect(m_widget, SIGNAL(textChanged()),
                   this,     SLOT(setModified()));
    }

    ReadWritePart::setReadWrite(rw);
}

void KCallGuiPart::setModified(bool modified)
{
    kdDebug() << "KCallGuiPart::setModified()" << endl;
    // get a handle on our Save action and make sure it is valid
    KAction *save = actionCollection()->action(KStdAction::stdName(KStdAction::Save));
    if (!save)
        return;

    // if so, we either enable or disable it based on the current
    // state
    if (modified)
        save->setEnabled(true);
    else
        save->setEnabled(false);

    // in any event, we want our parent to do it's thing
    ReadWritePart::setModified(modified);
}

bool KCallGuiPart::openFile()
{
    kdDebug() << "KCallGuiPart::openFile()" << endl;
    // m_file is always local so we can use QFile on it
    QFile file(m_file);
    if (file.open(IO_ReadOnly) == false)
        return false;

    // our example widget is text-based, so we use QTextStream instead
    // of a raw QDataStream
    QTextStream stream(&file);
    QString str;
    while (!stream.eof())
        str += stream.readLine() + "\n";

    file.close();

    // now that we have the entire file, display it
    //m_widget->setText(str);

    // just for fun, set the status bar
    emit setStatusBarText( m_url.prettyURL() );

    return true;
}

bool KCallGuiPart::saveFile()
{
    kdDebug() << "KCallGuiPart::saveFile()" << endl;
    // if we aren't read-write, return immediately
    if (isReadWrite() == false)
        return false;

    // m_file is always local, so we use QFile
    QFile file(m_file);
    if (file.open(IO_WriteOnly) == false)
        return false;

    // use QTextStream to dump the text to the file
    QTextStream stream(&file);
    stream << m_widget->text();

    file.close();

    return true;
}

void KCallGuiPart::fileOpen()
{
    kdDebug() << "KCallGuiPart::fileOpen()" << endl;
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    QString file_name = KFileDialog::getOpenFileName();

    if (file_name.isEmpty() == false)
        openURL(file_name);
}

void KCallGuiPart::fileSaveAs()
{
    kdDebug() << "KCallGuiPart::fileSaveAs()" << endl;
    // this slot is called whenever the File->Save As menu is selected,
    QString file_name = KFileDialog::getSaveFileName();
    if (file_name.isEmpty() == false)
        saveAs(file_name);
}

void KCallGuiPart::configure()
{
   if(KConfigDialog::showDialog("settings"))
     return;
   KConfigDialog *dialog = new KConfigDialog(mainWidget, "settings", KCallSettings::self(), KDialogBase::IconList);
   dialog->addPage(new SIPPrefs(), KCallSipSettings::self(), i18n("SIP"), "kcall");
   //dialog->addPage(new General(0, "General"), i18n("General") );
   //dialog->addPage(new Appearance(0, "Style"), i18n("Appearance") );
   connect(dialog, SIGNAL(settingsChanged()), mGuiCore, SLOT(loadSettings()));
   dialog->show();
}


void KCallGuiPart::initMainWidget()
{
    kdDebug() << "KCallGuiPart::initMainWidget()" << endl;
    
    //->kCallStatus();
}
















// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance*  KCallGuiPartFactory::s_instance = 0L;
KAboutData* KCallGuiPartFactory::s_about = 0L;

KCallGuiPartFactory::KCallGuiPartFactory()
    : KParts::Factory()
{
}

KCallGuiPartFactory::~KCallGuiPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KCallGuiPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
                                                        QObject *parent, const char *name,
                                                        const char *classname, const QStringList &args )
{
    // Create an instance of our Part
    KCallGuiPart* obj = new KCallGuiPart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
    if (QCString(classname) == "KParts::ReadOnlyPart")
        obj->setReadWrite(false);

    return obj;
}

KInstance* KCallGuiPartFactory::instance()
{
    if( !s_instance )
    {
        s_about = new KAboutData("kcallpart", I18N_NOOP("KCallPart"), "0.1");
        s_about->addAuthor("Mike Hauth", 0, "mike@mike-hauth.de");
        s_about->addAuthor("Eva Brucherseifer", 0, "eva.brucherseifer@basyskom.de");
        s_instance = new KInstance(s_about);
    }
    return s_instance;
}

extern "C"
{
    void* init_libkcallpart()
    {
        return new KCallGuiPartFactory;
    }
};

#include "kcallgui_part.moc"
