#include "kcallguicore.h"

#include <kdebug.h>

#include "kcallgui_part.h"
#include "kcallguimain.h"


KCallGuiCore::KCallGuiCore(KCallGuiPart* part, KCallGuiMain* widget)
    : QObject(), DCOPObject("KCallGuiCore")
{
    kdDebug() << "KCallGuiCore::KCallGuiCore()" << endl;

    pKCallPart = part;
    pKCallGui = widget;
    
    dcopClient = new DCOPClient();
    dcopClient->attach();
    
    registrateAtKCall(true);
    
    if(!getKCallStatus())
    {
        system("kcallapplet");
    }
}

KCallGuiCore::~KCallGuiCore()
{
    kdDebug() << "KCallGuiCore::~KCallGuiCore()" << endl;

    registrateAtKCall(false);
}

void KCallGuiCore::kCallStatusReport(bool online)
{
    kdDebug() << "KCallGuiCore::kCallStatusReport()" << endl;
    
    kCallStatus = online;
    pKCallGui->kCallStatus( kCallStatus );
}

bool KCallGuiCore::getKCallStatus()
{
    kdDebug() << "KCallGuiCore::checkKCallStatus()" << endl;
    
    bool s =  dcopClient->isApplicationRegistered("kcallapplet"); 
    pKCallGui->kCallStatus(s);
    return s;
     
}

bool KCallGuiCore::registrateAtKCall(bool state)
{
    kdDebug() << "KCallGuiCore::registrateAtKCall()" << endl;
        
    QByteArray  data;
    QDataStream senddata(data, IO_WriteOnly);
    senddata << state;         
    
    if(dcopClient->send("kcallapplet", "KCallCore", "guiStatusReport(bool)", data))
        return true;
    else
        return false;
}
 
QString KCallGuiCore::dial(QString num)
{
    kdDebug() << "KCallGuiCore::dial()" << endl;
    
    QCString reply_type;
    QByteArray data;
    QByteArray reply_data;
    QDataStream arg(data, IO_WriteOnly);
    
    arg << num;
    if(!dcopClient->call("kcallapplet", "KCallCore", "dial(QString)", data, reply_type, reply_data))
    {
        kdDebug() << "KCallGuiCore::dial():Error! No successfull sending" << endl;
        return false;
    }else
    {
        QDataStream answer(reply_data, IO_ReadOnly);
        if(reply_type == "QString")
        {
            QString callId;
            answer >> callId;
            pKCallGui->newCall(callId, num, "", 2);
            return callId;
        }else
        {
            kdDebug() << "KCallGuiCore::dial():Error! Wrong Reply Data Type" << endl;
            return false;
        }
    }
}

void KCallGuiCore::newCall(QString callid, QString caller, QString subject, int status)
{
    kdDebug() << "KCallGuiCore::newCall()" << endl;
    
    pKCallGui->newCall(callid, caller, subject, status);
}

void KCallGuiCore::cancelCall(QString callid)
{
    kdDebug() << "KCallGuiCore::cancelCall()" << endl;
    
    pKCallGui->cancelCall(callid);
}


void KCallGuiCore::showSettingsDialog()
{
    kdDebug() << "KCallGuiCore::showSettingsDialog()" << endl;
    pKCallPart->configure();
}


void KCallGuiCore::acceptCall(QString callid)
{
    kdDebug() << "KCallGuiCore::acceptCall()" << endl;
    
    dcopClient->send("kcallapplet", "KCallCore", "acceptCall(QString)", callid);
}

void KCallGuiCore::holdCall(QString callid)
{
    kdDebug() << "KCallGuiCore::holdCall()" << endl;
    
    dcopClient->send("kcallapplet", "KCallCore", "holdCall(QString)", callid);
}

void KCallGuiCore::rejectCall(QString callid)
{
    kdDebug() << "KCallGuiCore::rejectCall()" << endl;
    
    dcopClient->send("kcallapplet", "KCallCore", "rejectCall(QString)", callid);
}

void KCallGuiCore::hangupCall(QString callid)
{
    kdDebug() << "KCallGuiCore::hangupCall()" << endl;
    
    dcopClient->send("kcallapplet", "KCallCore", "hangUp(QString)", callid);
}

void KCallGuiCore::loadSettings()
{
    kdDebug() << "KCallGuiCore::loadSettings()" << endl;

    dcopClient->send("kcallapplet", "KCallCore", "settingsChanged()", "" );
}

#include "kcallguicore.moc"
