/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <kdebug.h>
#include <kabc/stdaddressbook.h>
#include <kabc/addressee.h>
#include <kplugininfo.h>
#include <kiconloader.h>
#include <kapplication.h>

#include <qregexp.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qobject.h>

#include "diallistreadonly.h"
#include "kcallguiconst.h"
#include "viscard.h"
#include "kcallsettings.h"
#include "kcallsipsettings.h"

struct CallDesc
{
    QString subject;
    QString callerUser;
    int status;//1 incoming, 2 outgoing, 3 rejected, 4 hangup, 5 canceled, 6 active
};

void KCallGuiMain::init()
{
    kdDebug() << "KCallGuiMain::init()" << endl;
    
    //load the history and completion list for nrInput
    QStringList nrInputHistoryList;
    nrInputHistoryList = KCallSettings::completion();
    kdDebug() << nrInputHistoryList << endl;
	nrInput->completionObject()->setItems( nrInputHistoryList );
    //nrInput->setHistoryItems( nrInputHistoryList, true ); //won´t work, so the dirty version
    
    for ( QStringList::Iterator it = nrInputHistoryList.begin(); it != nrInputHistoryList.end(); ++it ) {
        nrInput->addToHistory(*it);
    }
    
    //Pixmaps for states
    statusPix[1] = SmallIcon("forward");
    statusPix[2] = SmallIcon("back");
    statusPix[3] = SmallIcon("rejected");
    statusPix[4] = SmallIcon("hangedup");
    statusPix[5] = SmallIcon("canceled");
    statusPix[6] = SmallIcon("active");
    
    // init searchInput
    eraseButton->setPixmap(SmallIcon(QApplication::reverseLayout() ? "clear_left" : "locationbar_erase" ) );
    eraseButton->setAccel( QKeySequence( CTRL+ALT+Key_S ) );
    eraseButton->setAutoRaise( true );
    
    // init dialing
    acceptButton->setIconSet( SmallIconSet("phone_accept") );
    hangupButton->setIconSet( SmallIconSet("phone_reject") );
    //hangupButton->setIconSet( SmallIconSet("phone_reject",22 ) );

    //init actCalls-List
    actCalls->addColumn("Caller",200);
    actCalls->addColumn("Subject",0);
    callStatusCol = actCalls->addColumn("Status",60);
    callIdCol = actCalls->addColumn("CallId",0);
    actCallNrCol = actCalls->addColumn("CallNr",0);
    
    //init phonebookList        
    ab = KABC::StdAddressBook::self( true );
    phonebookList->addColumn("Name");
    pbLCallNrCol = phonebookList->addColumn("PhoneNumber");
    phonebookList->addColumn("ABId",0);
    setPhonebookList();
    
    connect(ab, SIGNAL(addressBookChanged(AddressBook*)), this, SLOT(setPhonebookList()));

    //init historyList
    KCallSipSettings::self()->config()->setGroup("CallLists");
    hl = new DialListReadOnly(KCallSipSettings::maxLength());
    historyList->addColumn("Contact");
    historyList->addColumn("Type");
    historyList->addColumn("Date");
    historyList->addColumn("Duration");
    historyList->addColumn("callId",0);
    hLCallNrCol = historyList->addColumn("CallNr",0);
    setHistoryList();
    
    visCardLayout = new QVBoxLayout( info, 11, 6, "visCardLayout");
    visitcard = new VisCard(info);
    visCardLayout->addWidget(visitcard);
    connect(visitcard, SIGNAL(dialNr(const QString&)), this, SLOT(dialByInput(const QString&)));

    phonebookList->setSelected(phonebookList->firstChild(), true); 
    
    setDialTab();
    
    //newCall("buh", "sip:244@192.168.2.230", "", 1);
}

void KCallGuiMain::destroy()
{
    kdDebug() << "KCallGuiMain::destroy()" << endl;
    
    QStringList nrInputHistoryList;
    nrInputHistoryList = nrInput->completionObject()->items();
    KCallSettings::setCompletion(nrInputHistoryList);
    nrInputHistoryList = nrInput->historyItems();
    KCallSettings::setHistory(nrInputHistoryList);
    KCallSettings::self()->writeConfig();
}


void KCallGuiMain::newCall(QString callid, QString caller, QString subject, int status)
{
    kdDebug() << "KCallGuiMain::newCall()" << endl;
    kdDebug() << callid << endl;
    CallDesc tmpDesc;
    tmpDesc.subject = subject;
    tmpDesc.callerUser = caller;
    //tmpDesc.callerProxy = "";
    tmpDesc.status = status;
    
    callsInList.insert(callid, tmpDesc);
    
    QString callerText = caller;
    KABC::Addressee person = getPersonFromNumber( caller );
    if(!person.isEmpty())
        callerText = person.formattedName();
        
    new KListViewItem( actCalls, callerText, subject, "", callid, caller );
    setCallStatus(callid, status);
    
    //setButtonsActCalls();
    //actCalls->insertItem(newcalltmp);
    
    
}

void KCallGuiMain::dialByInput(const QString &phoneNr)
{
    kdDebug() << "KCallGuiMain::dialByInput()" << endl;
    
    nrInput->insertURL(phoneNr);
    kdDebug() << "PhoneNumber" << phoneNr << endl;
    emit dial(phoneNr);
}

void KCallGuiMain::decideAcceptAction()
{
	kdDebug() << "KCallGuiMain::decideAcceptAction()" << endl;

	bool noActCalls = actCalls->selectedItems().isEmpty();
	if(noActCalls)
	{
		//kdDebug() << "Nummer aus Input:" << nrInput->currentText()<<endl;
		dialByInput(nrInput->currentText());
	}

}

void KCallGuiMain::acceptCall()
{
    kdDebug() << "KCallGuiMain::acceptCall()" << endl;
    
    
    QPtrList <QListViewItem> selectedCalls;
    selectedCalls = actCalls->selectedItems();
    
    QListViewItem *call;
    for ( call = selectedCalls.first(); call; call = selectedCalls.next() )
    {
        QString tmpCallId = call->text(callIdCol);
        
        setCallStatus(tmpCallId, 6);
        
        setButtonsActCalls();
        call->setText(callStatusCol, QString::number(callsInList[tmpCallId].status));
        emit accept(tmpCallId);        
    }
}

void KCallGuiMain::rejectCall()
{
    kdDebug() << "KCallGuiMain::rejectCall()" << endl;
        
    QPtrList <QListViewItem> selectedCalls;
    selectedCalls = actCalls->selectedItems();
    
    QListViewItem *call;
    for ( call = selectedCalls.first(); call; call = selectedCalls.next() )
    {
        QString tmpCallId = call->text(callIdCol);
        
        setCallStatus(tmpCallId, 3);
    
        setButtonsActCalls();
        emit reject(tmpCallId);
        
        actCalls->takeItem(call);
        
    }
    setHistoryList();
}

void KCallGuiMain::hangupCall()
{
    kdDebug() << "KCallGuiMain::hangupCall()" << endl;
    
    QPtrList <QListViewItem> selectedCalls;
    selectedCalls = actCalls->selectedItems();
    
    QListViewItem *call;
    for ( call = selectedCalls.first(); call; call = selectedCalls.next() )
    {
        QString tmpCallId = call->text(callIdCol);
        
        setCallStatus(tmpCallId, 4);
    
        setButtonsActCalls();
        emit hangup(tmpCallId);
        
        actCalls->takeItem(call);
        
    }
     setHistoryList();
}

void KCallGuiMain::cancelCall(QString callId)
{
    kdDebug() << "KCallGuiMain::cancelCall()" << endl;
    
    QListViewItem *call; 
    call = actCalls->findItem(callId, callIdCol);
    
    setCallStatus(callId, 5);
    
    setButtonsActCalls();
    
    if(call != 0)
    {
        actCalls->takeItem(call);
    }
    
    setHistoryList();
}

void KCallGuiMain::setCallStatus(QString callid, int status)
{
    kdDebug() << "KCallGuiMain::setCallStatus():" << callid << endl;
    
    callsInList[callid].status = status;
    QListViewItem *call = actCalls->findItem(callid,callIdCol, ExactMatch);
    if(call != 0)
    {
        call->setPixmap(callStatusCol, statusPix[status]);
    }
    
}

void KCallGuiMain::kCallStatus(bool online)
{
    kdDebug() << "KCallGuiMain::kCallStatus()" << endl;    
    
    nrInput->setEnabled(online);
    acceptButton->setEnabled(online);
    hangupButton->setEnabled(online);
}


void KCallGuiMain::setPhonebookList()
{
    kdDebug() << "KCallGuiMain::setPhonebookList()" << endl;    
        
    ab->load();
    KABC::AddressBook::ConstIterator it;
    
    phonebookList->clear();
    for ( it = ab->begin(); it != ab->end(); ++it ) 
    {
        KABC::PhoneNumber::List nrsOfPerson =  it->phoneNumbers();
        
        int sumOfNrs = nrsOfPerson.size();
        if(sumOfNrs > 0)
        {
            new KListViewItem(phonebookList, it->formattedName(), nrsOfPerson[0].number(), it->uid());	
        }
        for ( int i = 0; i < sumOfNrs; i++)
        {
            //new KListViewItem(phonebookList, it->formattedName(), nrsOfPerson[i].number(), it->uid());
        }
    }
}

void KCallGuiMain::setHistoryList()
{
    kdDebug() << "KCallGuiMain::setHistoryList()" << endl;

    historyList->clear();
    hl->load();
    
    QValueList<call> history = hl->getAllCalls();
    QValueList<call>::iterator it;
    for ( it = history.begin(); it != history.end(); ++it )
    {
        
        //Building the contact 
        QString contact = "--unknown--";
        QString phoneNr = "--unknown--";
        if((*it).callerNr != "") //if PhoneNr IS NOT empty
        {
            phoneNr = (*it).callerNr;
            KABC::Addressee person = getPersonFromNumber((*it).callerNr);
            if(person.isEmpty())//if PhoneNr IS NOT in Addressbook
            {
                if((*it).callerName != "") //if callerName IS NOT empty
                {
                    contact = (*it).callerName;
                }else   //if callerName IS empty
                {
                    contact = (*it).callerNr;
                } 
            }else //if PhoneNr IS in Addressbook
            {
                contact = person.formattedName();   
            }
        }else if((*it).callerName != "") //if PhoneNr IS empty but callerName IS NOT EMPTY 
        {
            contact = (*it).callerName;
        }else //if phoneNr AND callerName IS EMPTY
        {
            contact = "Unknown";
        }
        
        //Building the status
        int status      = (*it).type;
        
        //Building the Date and Time
        QString date = "";
        if(QDate::currentDate() != (*it).date)
        {
            date += (*it).date.toString("dd.MM.yyyy");
            date += " ";
        }
        date += (*it).time.toString("hh:mm:ss");
        
        //Building the Duration
        //QString duration    = QString::number((*it).duration);
        QDateTime durationTime;
        durationTime.setTime_t((*it).duration,Qt::UTC);
        
        int durInt = (*it).duration;
        QString duration = "00:00";
        if(durInt < 3600)
            duration = durationTime.toString("mm:ss");
        else if(durInt < 3600*24)
            duration = durationTime.toString("hh:mm:ss");
        else
            duration = durationTime.toString("dd:hh:mm:ss");
            
        //Building the CallId
        QString callId      = (*it).callId;
        
        KListViewItem *tmp = new KListViewItem(historyList, contact, "", date, duration, callId, phoneNr);
        tmp->setPixmap(1, statusPix[status]);
        
    }
}

KABC::Addressee KCallGuiMain::getPersonFromNumber(QString phoneNumber)
{
    kdDebug() << "KCallGuiMain::getPersonFromKABC(" << phoneNumber << ")" << endl;
    
    ab->load();
    
    QRegExp r( "[/*/-/ ]" );
    QString localPhone( phoneNumber );

    bool found = false;
    KABC::AddressBook::ConstIterator iter;
    KABC::PhoneNumber::List::ConstIterator phoneIter;
    KABC::PhoneNumber::List phoneList;
    for ( iter = ab->begin(); !found && ( iter != ab->end() ); ++iter ) 
    {
        phoneList = (*iter).phoneNumbers();
        for ( phoneIter = phoneList.begin(); !found && ( phoneIter != phoneList.end() ); ++phoneIter) 
        {
            // Get rid of separator chars so just the numbers are compared.
            if ( (*phoneIter).number().replace( r, "" ) == localPhone.replace( r, "" ) ) 
            {
                return ab->findByUid(iter->uid());
                //ownerName = (*iter).realName();
                found = true;
            }
        }
    }
    KABC::Addressee ad;
    return ad;

    
    /*
    ab->load();
    KABC::AddressBook::ConstIterator it; //is something like Addressee
    for ( it = ab->begin(); it != ab->end(); ++it ) 
    {
        KABC::PhoneNumber::List nrsOfPerson = it->phoneNumbers();
        
        int sumOfNrs = nrsOfPerson.size();
        
        for ( int i = 0; i < sumOfNrs; i++)
        {
            if(nrsOfPerson[i]==phoneNumber)
            {
                kdDebug() << it->formattedName() << endl;
                return ab->findByUid(it->uid());
            }
        }
    }
    KABC::Addressee ad;
    return ad;*/
}

void KCallGuiMain::setVisCard(QListViewItem* item)
{
    kdDebug() << "KCallGuiMain::setVisCard()" << endl;
    
    QString phoneNr = "";
    if(item != NULL )
    {
        if(item == actCalls->selectedItem())
        {
            phoneNr = item->text(actCallNrCol);
            phonebookList->clearSelection();
            historyList->clearSelection();
        }else
        if(item == historyList->selectedItem())
        {
            phoneNr = item->text(hLCallNrCol);
            actCalls->clearSelection();
            phonebookList->clearSelection();
        }else
        if(item == phonebookList->selectedItem())
        {
            phoneNr = item->text(pbLCallNrCol);
            actCalls->clearSelection();
            historyList->clearSelection();
        }
    }
    KABC::Addressee ad = getPersonFromNumber(phoneNr);
    
    visitcard->setByAddressee(ad);
    visitcard->show();
}

void KCallGuiMain::setDialTab()
{
    kdDebug() << "KCallGuiMain::setDialTab()" << endl;
    
//    if(nrTabs->currentPage() == phonebookTab)
    {
        searchInput->setListView(phonebookList);
        //setPhonebookList();
    }
    /*else
    if(callTabs->currentPage() == callListTab)
    {
        searchInput->setListView(historyList);
        //setHistoryList();
    }    
*/
}

void KCallGuiMain::setButtonsActCalls()
{
    kdDebug() << "KCallGuiMain::setButtonsActCalls()" << endl;
    
    QPtrList <QListViewItem> selectedCalls;
    selectedCalls = actCalls->selectedItems();
    
    QListViewItem *call;
    for ( call = selectedCalls.first(); call; call = selectedCalls.next() )
    {
        QString tmpCallId = call->text(callIdCol);
        int status = callsInList[tmpCallId].status;
        
        if(status == 1)
        {   
            disconnect(hangupButton, SIGNAL(clicked()), this, SLOT(hangupCall()));
               connect(hangupButton, SIGNAL(clicked()), this, SLOT(rejectCall()));
            hangupButton->setText("Reject");
            hangupButton->setEnabled(true);
            acceptButton->setEnabled(true);
        }
        else if(status == 2)
        {
            disconnect(hangupButton, SIGNAL(clicked()), this, SLOT(rejectCall()));
               connect(hangupButton, SIGNAL(clicked()), this, SLOT(hangupCall()));
            hangupButton->setText("HangUp");
            hangupButton->setEnabled(true);
            acceptButton->setEnabled(false);
        }
        else if(status == 6)
        {
            disconnect(hangupButton, SIGNAL(clicked()), this, SLOT(rejectCall()));
               connect(hangupButton, SIGNAL(clicked()), this, SLOT(hangupCall()));
            hangupButton->setText("HangUp");
            acceptButton->setEnabled(false);
        }
    }
}
